@@ -43,7 +43,7 @@ class ItemSync::Private
   public:
     Private( ItemSync *parent ) :
       q( parent ),
-      mTransactionMode( Single ),
+      mTransactionMode( SingleTransaction ),
       mCurrentTransaction( 0 ),
       mTransactionJobs( 0 ),
       mPendingJobs( 0 ),
@@ -78,13 +78,7 @@ class ItemSync::Private
     QHash<QString, Akonadi::Item> mLocalItemsByRemoteId;
     QSet<Akonadi::Item> mUnprocessedLocalItems;
 
-    // transaction mode, TODO: make this public API?
-    enum TransactionMode {
-      Single,
-      Chunkwise,
-      None
-    };
-    TransactionMode mTransactionMode;
+    ItemSync::TransactionMode mTransactionMode;
     TransactionSequence *mCurrentTransaction;
     int mTransactionJobs;
 
@@ -274,7 +268,12 @@ void ItemSync::Private::execute()
   if ( !mLocalListDone )
     return;
 
-  if ( (mTransactionMode == Single && !mCurrentTransaction) || mTransactionMode == Chunkwise ) {
+  // early exit to avoid unnecessary TransactionSequence creation in MultipleTransactions mode
+  // TODO: do the transaction handling in a nicer way instead, only creating TransactionSequences when really needed
+  if ( !mDeliveryDone && mRemoteItems.isEmpty() )
+    return;
+
+  if ( (mTransactionMode == SingleTransaction && !mCurrentTransaction) || mTransactionMode == MultipleTransactions) {
     ++mTransactionJobs;
     mCurrentTransaction = new TransactionSequence( q );
     mCurrentTransaction->setAutomaticCommittingEnabled( false );
@@ -283,7 +282,7 @@ void ItemSync::Private::execute()
 
   processItems();
   if ( !mDeliveryDone ) {
-    if ( mTransactionMode == Chunkwise && mCurrentTransaction ) {
+    if ( mTransactionMode == MultipleTransactions && mCurrentTransaction ) {
       mCurrentTransaction->commit();
       mCurrentTransaction = 0;
     }
@@ -337,6 +336,7 @@ void ItemSync::Private::processItems()
       remoteItem.setSize( localItem.size() );
       remoteItem.setRemoteId( localItem.remoteId() );  // in case someone clears remoteId by accident
       ItemModifyJob *mod = new ItemModifyJob( remoteItem, subjobParent() );
+      mod->disableRevisionCheck();
       q->connect( mod, SIGNAL( result( KJob* ) ), q, SLOT( slotLocalChangeDone( KJob* ) ) );
     } else {
       mProgress++;
@@ -360,8 +360,8 @@ void ItemSync::Private::deleteItems( const Item::List &items )
 
     if ( !delItem.isValid() ) {
 #ifndef NDEBUG
-      kWarning() << "Delete item (remoteeId=" << delItem.remoteId()
-                 << "mimeType=" << delItem.mimeType()
+      kWarning() << "Delete item (remoteeId=" << item.remoteId()
+                 << "mimeType=" << item.mimeType()
                  << ") does not have a valid UID and no item with that remote ID exists either";
 #endif
       continue;
@@ -418,7 +418,7 @@ void ItemSync::Private::slotTransactionResult( KJob *job )
 
 Job * ItemSync::Private::subjobParent() const
 {
-  if ( mCurrentTransaction && mTransactionMode != None )
+  if ( mCurrentTransaction && mTransactionMode != NoTransaction )
     return mCurrentTransaction;
   return q;
 }
@@ -459,5 +459,10 @@ void ItemSync::rollback()
   d->execute(); // end this in an ordered way, since we have an error set no real change will be done
 }
 
+void ItemSync::setTransactionMode(ItemSync::TransactionMode mode)
+{
+  d->mTransactionMode = mode;
+}
+
 
 #include "itemsync.moc"
