@@ -21,6 +21,7 @@
 
 #include "job.h"
 #include "job_p.h"
+#include "dbusconnectionpool.h"
 #include <QTime>
 #include "imapparser_p.h"
 #include "session.h"
@@ -103,14 +104,14 @@ void JobPrivate::init( QObject *parent )
     // Let's only check for the debugging console every 3 seconds, otherwise every single job
     // makes a dbus call to the dbus daemon, doesn't help performance.
     static QTime s_lastTime;
-    if ( s_lastTime.isNull() )
-      s_lastTime.start();
-    if ( s_lastTime.elapsed() > 3000 ) {
-      if ( QDBusConnection::sessionBus().interface()->isServiceRegistered(QLatin1String( "org.kde.akonadiconsole" ) ) ) {
+    if ( s_lastTime.isNull() || s_lastTime.elapsed() > 3000 ) {
+      if ( s_lastTime.isNull() )
+        s_lastTime.start();
+      if ( DBusConnectionPool::threadConnection().interface()->isServiceRegistered(QLatin1String( "org.kde.akonadiconsole" ) ) ) {
         s_jobtracker = new QDBusInterface( QLatin1String( "org.kde.akonadiconsole" ),
                                            QLatin1String( "/jobtracker" ),
                                            QLatin1String( "org.freedesktop.Akonadi.JobTracker" ),
-                                           QDBusConnection::sessionBus(), 0 );
+                                           DBusConnectionPool::threadConnection(), 0 );
       } else {
         s_lastTime.restart();
       }
@@ -208,6 +209,29 @@ void JobPrivate::writeData( const QByteArray & data )
   Q_ASSERT_X( !mWriteFinished, "Job::writeData()", "Calling writeData() after emitting writeFinished()" );
   mSession->d->writeData( data );
 }
+
+void JobPrivate::itemRevisionChanged( Akonadi::Item::Id itemId, int oldRevision, int newRevision )
+{
+  mSession->d->itemRevisionChanged( itemId, oldRevision, newRevision );
+}
+
+void JobPrivate::updateItemRevision( Akonadi::Item::Id itemId, int oldRevision, int newRevision )
+{
+  Q_Q( Job );
+  foreach ( KJob *j, q->subjobs() ) {
+    Akonadi::Job *job = qobject_cast<Akonadi::Job*>( j );
+    if ( job )
+      job->d_ptr->updateItemRevision( itemId, oldRevision, newRevision );
+  }
+  doUpdateItemRevision( itemId, oldRevision, newRevision );
+}
+
+void JobPrivate::doUpdateItemRevision( Akonadi::Item::Id itemId, int oldRevision, int newRevision )
+{
+  Q_UNUSED( itemId );
+  Q_UNUSED( oldRevision );
+  Q_UNUSED( newRevision );
+}
 //@endcond
 
 
