@@ -32,11 +32,10 @@
 #include <QtCore/QDebug>
 #include <QtCore/QTimer>
 #include "collectionfetchscope.h"
+#include <iterator>
 
 using namespace Akonadi;
 
-#define d d_ptr
-
 Monitor::Monitor( QObject *parent ) :
     QObject( parent ),
     d_ptr( new MonitorPrivate( this ) )
@@ -50,16 +49,21 @@ Monitor::Monitor(MonitorPrivate * d, QObject *parent) :
     QObject( parent ),
     d_ptr( d )
 {
+  d_ptr->init();
+  d_ptr->connectToNotificationManager();
 }
 //@endcond
 
 Monitor::~Monitor()
 {
-  delete d;
+  // :TODO: Unsubscribe from the notification manager. That means having some kind of reference
+  // counting on the server side.
+  delete d_ptr;
 }
 
 void Monitor::setCollectionMonitored( const Collection &collection, bool monitored )
 {
+  Q_D( Monitor );
   if ( monitored ) {
     d->collections << collection;
   } else {
@@ -71,6 +75,7 @@ void Monitor::setCollectionMonitored( const Collection &collection, bool monitor
 
 void Monitor::setItemMonitored( const Item & item, bool monitored )
 {
+  Q_D( Monitor );
   if ( monitored ) {
     d->items.insert( item.id() );
   } else {
@@ -82,6 +87,7 @@ void Monitor::setItemMonitored( const Item & item, bool monitored )
 
 void Monitor::setResourceMonitored( const QByteArray & resource, bool monitored )
 {
+  Q_D( Monitor );
   if ( monitored ) {
     d->resources.insert( resource );
   } else {
@@ -93,6 +99,7 @@ void Monitor::setResourceMonitored( const QByteArray & resource, bool monitored
 
 void Monitor::setMimeTypeMonitored( const QString & mimetype, bool monitored )
 {
+  Q_D( Monitor );
   if ( monitored ) {
     d->mimetypes.insert( mimetype );
   } else {
@@ -105,6 +112,7 @@ void Monitor::setMimeTypeMonitored( const QString & mimetype, bool monitored )
 
 void Akonadi::Monitor::setAllMonitored( bool monitored )
 {
+  Q_D( Monitor );
   d->monitorAll = monitored;
 
   if ( !monitored ) {
@@ -116,67 +124,89 @@ void Akonadi::Monitor::setAllMonitored( bool monitored )
 
 void Monitor::ignoreSession(Session * session)
 {
+  Q_D( Monitor );
   d->sessions << session->sessionId();
   connect( session, SIGNAL( destroyed( QObject* ) ), this, SLOT( slotSessionDestroyed( QObject* ) ) );
 }
 
 void Monitor::fetchCollection(bool enable)
 {
+  Q_D( Monitor );
   d->fetchCollection = enable;
 }
 
 void Monitor::fetchCollectionStatistics(bool enable)
 {
+  Q_D( Monitor );
   d->fetchCollectionStatistics = enable;
 }
 
 void Monitor::setItemFetchScope( const ItemFetchScope &fetchScope )
 {
+  Q_D( Monitor );
   d->mItemFetchScope = fetchScope;
 }
 
 ItemFetchScope &Monitor::itemFetchScope()
 {
+  Q_D( Monitor );
   return d->mItemFetchScope;
 }
 
 void Monitor::setCollectionFetchScope( const CollectionFetchScope &fetchScope )
 {
+  Q_D( Monitor );
   d->mCollectionFetchScope = fetchScope;
 }
 
 CollectionFetchScope& Monitor::collectionFetchScope()
 {
+  Q_D( Monitor );
   return d->mCollectionFetchScope;
 }
 
 Collection::List Monitor::collectionsMonitored() const
 {
+  Q_D( const Monitor );
   return d->collections;
 }
 
 QList<Item::Id> Monitor::itemsMonitored() const
 {
+  Q_D( const Monitor );
   return d->items.toList();
 }
 
+QVector<Item::Id> Monitor::itemsMonitoredEx() const
+{
+  Q_D( const Monitor );
+  QVector<Item::Id> result;
+  result.reserve( d->items.size() );
+  qCopy( d->items.begin(), d->items.end(), std::back_inserter( result ) );
+  return result;
+}
+
 QStringList Monitor::mimeTypesMonitored() const
 {
+  Q_D( const Monitor );
   return d->mimetypes.toList();
 }
 
 QList<QByteArray> Monitor::resourcesMonitored() const
 {
+  Q_D( const Monitor );
   return d->resources.toList();
 }
 
 bool Monitor::isAllMonitored() const
 {
+  Q_D( const Monitor );
   return d->monitorAll;
 }
 
 void Monitor::setSession( Akonadi::Session *session )
 {
+  Q_D( Monitor );
   if (session == d->session)
     return;
 
@@ -191,9 +221,8 @@ void Monitor::setSession( Akonadi::Session *session )
 
 Session* Monitor::session() const
 {
+  Q_D( const Monitor );
   return d->session;
 }
 
-#undef d
-
 #include "monitor.moc"
