@@ -23,11 +23,14 @@
 
 #include "collectionfetchjob.h"
 #include "collectionstatistics.h"
+#include "dbusconnectionpool.h"
 #include "itemfetchjob.h"
 #include "notificationmessage_p.h"
+#include "notificationmanagerinterface.h"
 #include "session.h"
 
 #include <kdebug.h>
+#include <kcomponentdata.h>
 
 using namespace Akonadi;
 
@@ -35,7 +38,7 @@ static const int PipelineSize = 5;
 
 MonitorPrivate::MonitorPrivate(Monitor * parent) :
   q_ptr( parent ),
-  nm( 0 ),
+  notificationSource( 0 ),
   monitorAll( false ),
   session( Session::defaultSession() ),
   collectionCache( 3*PipelineSize, session ), // needs to be at least 3x pipeline size for the collection move case
@@ -50,27 +53,52 @@ void MonitorPrivate::init()
 {
   QObject::connect( &collectionCache, SIGNAL( dataAvailable() ), q_ptr, SLOT( dataAvailable() ) );
   QObject::connect( &itemCache, SIGNAL( dataAvailable() ), q_ptr, SLOT( dataAvailable() ) );
+  QObject::connect( ServerManager::self(), SIGNAL(stateChanged(Akonadi::ServerManager::State)),
+                    q_ptr, SLOT(serverStateChanged(Akonadi::ServerManager::State)) );
+
+  NotificationMessage::registerDBusTypes();
+
+  statisticsCompressionTimer.setSingleShot( true );
+  statisticsCompressionTimer.setInterval( 500 );
+  QObject::connect( &statisticsCompressionTimer, SIGNAL(timeout()), q_ptr, SLOT(slotFlushRecentlyChangedCollections()) );
 }
 
 bool MonitorPrivate::connectToNotificationManager()
 {
-  NotificationMessage::registerDBusTypes();
+  delete notificationSource;
+  notificationSource = 0;
 
-  if ( !nm )
-    nm = new org::freedesktop::Akonadi::NotificationManager( QLatin1String( "org.freedesktop.Akonadi" ),
-                                                     QLatin1String( "/notifications" ),
-                                                     QDBusConnection::sessionBus(), q_ptr );
-  else
-    return true;
+  org::freedesktop::Akonadi::NotificationManager manager(
+          QLatin1String( "org.freedesktop.Akonadi" ),
+          QLatin1String( "/notifications" ),
+          DBusConnectionPool::threadConnection() );
 
-  if ( !nm ) {
-    kWarning() << "Unable to connect to notification manager";
-  } else {
-    QObject::connect( nm, SIGNAL( notify( Akonadi::NotificationMessage::List ) ),
-                      q_ptr, SLOT( slotNotify( Akonadi::NotificationMessage::List ) ) );
-    return true;
+  QDBusObjectPath p = manager.subscribe( KGlobal::mainComponent().componentName() );
+  if ( manager.lastError().isValid() ) {
+    // :TODO: What to do?
+    return false;
   }
-  return false;
+
+  notificationSource = new org::freedesktop::Akonadi::NotificationSource(
+              QLatin1String( "org.freedesktop.Akonadi" ),
+              p.path(),
+              DBusConnectionPool::threadConnection(), q_ptr );
+
+  if ( !notificationSource ) {
+    // :TODO: error handling
+    return false;
+  }
+
+  QObject::connect( notificationSource, SIGNAL( notify( Akonadi::NotificationMessage::List ) ),
+                    q_ptr, SLOT( slotNotify( Akonadi::NotificationMessage::List ) ) );
+
+  return true;
+}
+
+void MonitorPrivate::serverStateChanged(ServerManager::State state)
+{
+  if ( state == ServerManager::Running )
+    connectToNotificationManager();
 }
 
 int MonitorPrivate::pipelineSize() const
@@ -224,7 +252,7 @@ bool MonitorPrivate::ensureDataAvailable( const NotificationMessage &msg )
   return allCached;
 }
 
-void MonitorPrivate::emitNotification( const NotificationMessage &msg )
+bool MonitorPrivate::emitNotification( const NotificationMessage &msg )
 {
   const Collection parent = collectionCache.retrieve( msg.parentCollection() );
   Collection destParent;
@@ -233,11 +261,13 @@ void MonitorPrivate::emitNotification( const NotificationMessage &msg )
 
   if ( msg.type() == NotificationMessage::Collection ) {
     const Collection col = collectionCache.retrieve( msg.uid() );
-    emitCollectionNotification( msg, col, parent, destParent );
+    return emitCollectionNotification( msg, col, parent, destParent );
   } else if ( msg.type() == NotificationMessage::Item ) {
     const Item item = itemCache.retrieve( msg.uid() );
-    emitItemNotification( msg, item, parent, destParent );
+    return emitItemNotification( msg, item, parent, destParent );
   }
+
+  return false; // nothing emitted
 }
 
 void MonitorPrivate::dataAvailable()
@@ -342,8 +372,8 @@ void MonitorPrivate::slotNotify( const NotificationMessage::List &msgs )
   dispatchNotifications();
 }
 
-void MonitorPrivate::emitItemNotification( const NotificationMessage &msg, const Item &item,
-                                             const Collection &collection, const Collection &collectionDest  )
+bool MonitorPrivate::emitItemNotification( const NotificationMessage &msg, const Item &item,
+                                           const Collection &collection, const Collection &collectionDest  )
 {
   Q_ASSERT( msg.type() == NotificationMessage::Item );
   Collection col = collection;
@@ -358,6 +388,7 @@ void MonitorPrivate::emitItemNotification( const NotificationMessage &msg, const
     if ( !msg.itemParts().isEmpty() )
       colDest.setResource( QString::fromLatin1( *(msg.itemParts().begin()) ) );
   }
+
   Item it = item;
   if ( !it.isValid() || msg.operation() == NotificationMessage::Remove ) {
     it = Item( msg.uid() );
@@ -367,7 +398,12 @@ void MonitorPrivate::emitItemNotification( const NotificationMessage &msg, const
     // recover RID, in case of inter-resource moves the source RID is only in the
     // notification but not in the item loaded from Akonadi
     it.setRemoteId( msg.remoteId() );
+  } else if ( msg.operation() == NotificationMessage::Move && col.resource() != colDest.resource() ) {
+    // recover RID in case of inter-resource moves (part 2), if the destination has already
+    // changed the RID we need to reset to the one belonging to the source resource
+    it.setRemoteId( msg.remoteId() );
   }
+
   if ( !it.parentCollection().isValid() ) {
     if ( msg.operation() == NotificationMessage::Move )
       it.setParentCollection( colDest );
@@ -386,40 +422,46 @@ void MonitorPrivate::emitItemNotification( const NotificationMessage &msg, const
 
   switch ( msg.operation() ) {
     case NotificationMessage::Add:
+      if ( q_ptr->receivers( SIGNAL(itemAdded(Akonadi::Item,Akonadi::Collection)) ) == 0 )
+        return false;
       emit q_ptr->itemAdded( it, col );
-      break;
+      return true;
     case NotificationMessage::Modify:
+      if ( q_ptr->receivers( SIGNAL(itemChanged(Akonadi::Item,QSet<QByteArray>)) ) == 0 )
+        return false;
       emit q_ptr->itemChanged( it, msg.itemParts() );
-      break;
+      return true;
     case NotificationMessage::Move:
+      if ( q_ptr->receivers( SIGNAL(itemMoved(Akonadi::Item,Akonadi::Collection,Akonadi::Collection)) ) == 0 )
+        return false;
       emit q_ptr->itemMoved( it, col, colDest );
-      break;
+      return true;
     case NotificationMessage::Remove:
+      if ( q_ptr->receivers( SIGNAL(itemRemoved(Akonadi::Item)) ) == 0 )
+        return false;
       emit q_ptr->itemRemoved( it );
-      break;
+      return true;
     case NotificationMessage::Link:
+      if ( q_ptr->receivers( SIGNAL(itemLinked(Akonadi::Item,Akonadi::Collection)) ) == 0 )
+        return false;
       emit q_ptr->itemLinked( it, col );
-      break;
+      return true;
     case NotificationMessage::Unlink:
+      if ( q_ptr->receivers( SIGNAL(itemUnlinked(Akonadi::Item,Akonadi::Collection)) ) == 0 )
+        return false;
       emit q_ptr->itemUnlinked( it, col );
-      break;
+      return true;;
     default:
       kDebug() << "Unknown operation type" << msg.operation() << "in item change notification";
-      break;
   }
+
+  return false;
 }
 
-void MonitorPrivate::emitCollectionNotification( const NotificationMessage &msg, const Collection &col,
-                                                   const Collection &par, const Collection &dest )
+bool MonitorPrivate::emitCollectionNotification( const NotificationMessage &msg, const Collection &col,
+                                                 const Collection &par, const Collection &dest )
 {
   Q_ASSERT( msg.type() == NotificationMessage::Collection );
-  Collection collection = col;
-  if ( !collection.isValid() || msg.operation() == NotificationMessage::Remove ) {
-    collection = Collection( msg.uid() );
-    collection.setResource( QString::fromUtf8( msg.resource() ) );
-    collection.setRemoteId( msg.remoteId() );
-  }
-
   Collection parent = par;
   if ( !parent.isValid() )
     parent = Collection( msg.parentCollection() );
@@ -427,6 +469,21 @@ void MonitorPrivate::emitCollectionNotification( const NotificationMessage &msg,
   if ( !destination.isValid() )
     destination = Collection( msg.parentDestCollection() );
 
+  Collection collection = col;
+  if ( !collection.isValid() || msg.operation() == NotificationMessage::Remove ) {
+    collection = Collection( msg.uid() );
+    collection.setResource( QString::fromUtf8( msg.resource() ) );
+    collection.setRemoteId( msg.remoteId() );
+  } else if ( collection.remoteId().isEmpty() && !msg.remoteId().isEmpty() ) {
+    // recover RID, in case of inter-resource moves the source RID is only in the
+    // notification but not in the item loaded from Akonadi
+    collection.setRemoteId( msg.remoteId() );
+  } else if ( msg.operation() == NotificationMessage::Move && parent.resource() != destination.resource() ) {
+    // recover RID in case of inter-resource moves (part 2), if the destination has already
+    // changed the RID we need to reset to the one belonging to the source resource
+    collection.setRemoteId( msg.remoteId() );
+  }
+
   if ( !collection.parentCollection().isValid() ) {
     if ( msg.operation() == NotificationMessage::Move )
       collection.setParentCollection( destination );
@@ -445,21 +502,44 @@ void MonitorPrivate::emitCollectionNotification( const NotificationMessage &msg,
 
   switch ( msg.operation() ) {
     case NotificationMessage::Add:
+      if ( q_ptr->receivers( SIGNAL(collectionAdded(Akonadi::Collection,Akonadi::Collection)) ) == 0 )
+        return false;
       emit q_ptr->collectionAdded( collection, parent );
-      break;
+      return true;
     case NotificationMessage::Modify:
+      if ( q_ptr->receivers( SIGNAL(collectionChanged(Akonadi::Collection)) ) == 0
+        && q_ptr->receivers( SIGNAL(collectionChanged(Akonadi::Collection,QSet<QByteArray>)) ) == 0 )
+        return false;
       emit q_ptr->collectionChanged( collection );
       emit q_ptr->collectionChanged( collection, msg.itemParts() );
-      break;
+      return true;
     case NotificationMessage::Move:
+      if ( q_ptr->receivers( SIGNAL(collectionMoved(Akonadi::Collection,Akonadi::Collection,Akonadi::Collection)) ) == 0 )
+        return false;
       emit q_ptr->collectionMoved( collection, parent, destination );
-      break;
+      return true;
     case NotificationMessage::Remove:
+      if ( q_ptr->receivers( SIGNAL(collectionRemoved(Akonadi::Collection)) ) == 0 )
+        return false;
       emit q_ptr->collectionRemoved( collection );
-      break;
+      return true;
+    case NotificationMessage::Subscribe:
+      if ( q_ptr->receivers( SIGNAL(collectionSubscribed(Akonadi::Collection,Akonadi::Collection)) ) == 0 )
+        return false;
+      if ( !monitorAll ) // ### why??
+        emit q_ptr->collectionSubscribed( collection, parent );
+      return true;
+    case NotificationMessage::Unsubscribe:
+      if ( q_ptr->receivers( SIGNAL(collectionUnsubscribed(Akonadi::Collection)) ) == 0 )
+        return false;
+      if ( !monitorAll ) // ### why??
+        emit q_ptr->collectionUnsubscribed( collection );
+      return true;
     default:
       kDebug() << "Unknown operation type" << msg.operation() << "in collection change notification";
   }
+
+  return false;
 }
 
 void MonitorPrivate::invalidateCaches( const NotificationMessage &msg )
@@ -474,7 +554,10 @@ void MonitorPrivate::invalidateCaches( const NotificationMessage &msg )
   }
 
   // modify removes the cache entry, as we need to re-fetch
-  if ( msg.operation() == NotificationMessage::Modify || msg.operation() == NotificationMessage::Move ) {
+  // And subscription modify the visibility of the collection by the collectionFetchScope.
+  if ( msg.operation() == NotificationMessage::Modify
+        || msg.operation() == NotificationMessage::Move
+        || msg.operation() == NotificationMessage::Subscribe ) {
     if ( msg.type() == NotificationMessage::Collection ) {
       collectionCache.update( msg.uid(), mCollectionFetchScope );
     } else if ( msg.type() == NotificationMessage::Item ) {
@@ -517,14 +600,14 @@ void MonitorPrivate::PurgeBuffer::purge( Collection::Id id )
   {
     if ( idx < 0 )
       break;
-    m_buffer.removeAt( idx );
+    m_buffer.remove( idx );
     if ( m_index > 0 )
       --m_index;
     idx = m_buffer.indexOf( id, idx );
   }
   while ( int idx = m_buffer.indexOf( id, m_index ) > -1 )
   {
-    m_buffer.removeAt( idx );
+    m_buffer.remove( idx );
   }
 }
 
@@ -537,7 +620,8 @@ Collection::Id MonitorPrivate::PurgeBuffer::buffer( Collection::Id id )
   Collection::Id bumpedId = -1;
   if ( m_buffer.size() == MAXBUFFERSIZE )
   {
-    bumpedId = m_buffer.takeAt( m_index );
+    bumpedId = m_buffer[ m_index ];
+    m_buffer.remove( m_index );
   }
 
   // Ensure that we don't put a duplicate @p id into the buffer.
@@ -549,4 +633,12 @@ Collection::Id MonitorPrivate::PurgeBuffer::buffer( Collection::Id id )
   return bumpedId;
 }
 
+void MonitorPrivate::notifyCollectionStatisticsWatchers(Entity::Id collection, const QByteArray& resource) {
+  if ( collection > 0 && (monitorAll || isCollectionMonitored( collection ) || resources.contains( resource ) ) ) {
+    recentlyChangedCollections.insert( collection );
+    if ( !statisticsCompressionTimer.isActive() )
+      statisticsCompressionTimer.start();
+  }
+}
+
 // @endcond
