@@ -24,6 +24,7 @@
 #include "resourceadaptor.h"
 #include "collectiondeletejob.h"
 #include "collectionsync_p.h"
+#include "dbusconnectionpool.h"
 #include "itemsync.h"
 #include "resourcescheduler_p.h"
 #include "tracerinterface.h"
@@ -67,26 +68,37 @@ class Akonadi::ResourceBasePrivate : public AgentBasePrivate
       : AgentBasePrivate( parent ),
         scheduler( 0 ),
         mItemSyncer( 0 ),
+        mItemSyncFetchScope( 0 ),
+        mItemTransactionMode( ItemSync::SingleTransaction ),
         mCollectionSyncer( 0 ),
-        mHierarchicalRid( false )
+        mHierarchicalRid( false ),
+        mUnemittedProgress( 0 )
     {
       Internal::setClientType( Internal::Resource );
       mStatusMessage = defaultReadyMessage();
+      mProgressEmissionCompressor.setInterval( 1000 );
+      mProgressEmissionCompressor.setSingleShot( true );
+    }
 
-      QDBusConnection::sessionBus().registerObject( QLatin1String( "/Debug" ), this, QDBusConnection::ExportScriptableSlots );
+    ~ResourceBasePrivate()
+    {
+      delete mItemSyncFetchScope;
     }
 
     Q_DECLARE_PUBLIC( ResourceBase )
 
     void delayedInit()
     {
-      if ( !QDBusConnection::sessionBus().registerService( QLatin1String( "org.freedesktop.Akonadi.Resource." ) + mId ) ) {
-        QString reason = QDBusConnection::sessionBus().lastError().message();
+      if ( !DBusConnectionPool::threadConnection().registerService( QLatin1String( "org.freedesktop.Akonadi.Resource." ) + mId ) ) {
+        QString reason = DBusConnectionPool::threadConnection().lastError().message();
         if ( reason.isEmpty() ) {
           reason = QString::fromLatin1( "this service is probably running already." );
         }
         kError() << "Unable to register service at D-Bus: " << reason;
-        QCoreApplication::instance()->exit(1);
+
+        if ( QThread::currentThread() == QCoreApplication::instance()->thread() )
+          QCoreApplication::instance()->exit(1);
+
       } else {
         AgentBasePrivate::delayedInit();
       }
@@ -100,15 +112,21 @@ class Akonadi::ResourceBasePrivate : public AgentBasePrivate
       scheduler->taskDone();
     }
 
+    void slotAbortRequested();
+
     void slotDeliveryDone( KJob* job );
     void slotCollectionSyncDone( KJob *job );
     void slotLocalListDone( KJob *job );
     void slotSynchronizeCollection( const Collection &col );
     void slotCollectionListDone( KJob *job );
+    void slotSynchronizeCollectionAttributes( const Collection &col );
+    void slotCollectionListForAttributesDone( KJob *job );
+    void slotCollectionAttributesSyncDone( KJob *job );
 
     void slotItemSyncDone( KJob *job );
 
     void slotPercent( KJob* job, unsigned long percent );
+    void slotDelayedEmitProgress();
     void slotDeleteResourceCollection();
     void slotDeleteResourceCollectionDone( KJob *job );
     void slotCollectionDeletionDone( KJob *job );
@@ -118,6 +136,30 @@ class Akonadi::ResourceBasePrivate : public AgentBasePrivate
 
     void changeCommittedResult( KJob* job );
 
+    void slotSessionReconnected()
+    {
+      Q_Q( ResourceBase );
+
+      new ResourceSelectJob( q->identifier() );
+    }
+
+    void createItemSyncInstanceIfMissing()
+    {
+      Q_Q( ResourceBase );
+      Q_ASSERT_X( scheduler->currentTask().type == ResourceScheduler::SyncCollection,
+                  "createItemSyncInstance", "Calling items retrieval methods although no item retrieval is in progress" );
+      if ( !mItemSyncer ) {
+        mItemSyncer = new ItemSync( q->currentCollection() );
+        mItemSyncer->setTransactionMode( mItemTransactionMode );
+        if ( mItemSyncFetchScope )
+          mItemSyncer->setFetchScope( *mItemSyncFetchScope );
+        mItemSyncer->setProperty( "collection", QVariant::fromValue( q->currentCollection() ) );
+        connect( mItemSyncer, SIGNAL( percent( KJob*, unsigned long ) ), q, SLOT( slotPercent( KJob*, unsigned long ) ) );
+        connect( mItemSyncer, SIGNAL( result( KJob* ) ), q, SLOT( slotItemSyncDone( KJob* ) ) );
+      }
+      Q_ASSERT( mItemSyncer );
+    }
+
   public Q_SLOTS:
     Q_SCRIPTABLE void dump()
     {
@@ -135,8 +177,13 @@ class Akonadi::ResourceBasePrivate : public AgentBasePrivate
 
     ResourceScheduler *scheduler;
     ItemSync *mItemSyncer;
+    ItemFetchScope *mItemSyncFetchScope;
+    ItemSync::TransactionMode mItemTransactionMode;
     CollectionSync *mCollectionSyncer;
     bool mHierarchicalRid;
+    QTimer mProgressEmissionCompressor;
+    int mUnemittedProgress;
+    QMap<Akonadi::Collection::Id, QVariantMap> mUnemittedAdvancedStatus;
 };
 
 ResourceBase::ResourceBase( const QString & id )
@@ -160,6 +207,8 @@ ResourceBase::ResourceBase( const QString & id )
            SLOT( retrieveCollections() ) );
   connect( d->scheduler, SIGNAL( executeCollectionSync( const Akonadi::Collection& ) ),
            SLOT( slotSynchronizeCollection( const Akonadi::Collection& ) ) );
+  connect( d->scheduler, SIGNAL( executeCollectionAttributesSync( const Akonadi::Collection& ) ),
+           SLOT( slotSynchronizeCollectionAttributes(Akonadi::Collection)) );
   connect( d->scheduler, SIGNAL( executeItemFetch( const Akonadi::Item&, const QSet<QByteArray>& ) ),
            SLOT( slotPrepareItemRetrieval(Akonadi::Item)) );
   connect( d->scheduler, SIGNAL( executeResourceCollectionDeletion() ),
@@ -172,15 +221,23 @@ ResourceBase::ResourceBase( const QString & id )
   connect( d->mChangeRecorder, SIGNAL( nothingToReplay() ), d->scheduler, SLOT( taskDone() ) );
   connect( d->mChangeRecorder, SIGNAL( collectionRemoved( const Akonadi::Collection& ) ),
            d->scheduler, SLOT( collectionRemoved( const Akonadi::Collection& ) ) );
+  connect( this, SIGNAL( abortRequested() ), this, SLOT( slotAbortRequested() ) );
   connect( this, SIGNAL( synchronized() ), d->scheduler, SLOT( taskDone() ) );
   connect( this, SIGNAL( agentNameChanged( const QString& ) ),
            this, SIGNAL( nameChanged( const QString& ) ) );
 
+  connect( &d->mProgressEmissionCompressor, SIGNAL( timeout() ),
+           this, SLOT( slotDelayedEmitProgress() ) );
+
   d->scheduler->setOnline( d->mOnline );
   if ( !d->mChangeRecorder->isEmpty() )
     d->scheduler->scheduleChangeReplay();
 
+  DBusConnectionPool::threadConnection().registerObject( QLatin1String( "/Debug" ), d, QDBusConnection::ExportScriptableSlots );
+
   new ResourceSelectJob( identifier() );
+
+  connect( d->mChangeRecorder->session(), SIGNAL( reconnected() ), SLOT( slotSessionReconnected() ) );
 }
 
 ResourceBase::~ResourceBase()
@@ -247,6 +304,14 @@ int ResourceBase::init( ResourceBase *r )
   return rv;
 }
 
+void ResourceBasePrivate::slotAbortRequested()
+{
+  Q_Q( ResourceBase );
+
+  scheduler->cancelQueues();
+  QMetaObject::invokeMethod( q, "abortActivity" );
+}
+
 void ResourceBase::itemRetrieved( const Item &item )
 {
   Q_D( ResourceBase );
@@ -282,6 +347,31 @@ void ResourceBasePrivate::slotDeliveryDone(KJob * job)
   scheduler->taskDone();
 }
 
+void ResourceBase::collectionAttributesRetrieved( const Collection &collection )
+{
+  Q_D( ResourceBase );
+  Q_ASSERT( d->scheduler->currentTask().type == ResourceScheduler::SyncCollectionAttributes );
+  if ( !collection.isValid() ) {
+    emit attributesSynchronized( d->scheduler->currentTask().collection.id() );
+    d->scheduler->taskDone();
+    return;
+  }
+
+  CollectionModifyJob *job = new CollectionModifyJob( collection );
+  connect( job, SIGNAL( result( KJob* ) ), SLOT( slotCollectionAttributesSyncDone( KJob* ) ) );
+}
+
+void ResourceBasePrivate::slotCollectionAttributesSyncDone(KJob * job)
+{
+  Q_Q( ResourceBase );
+  Q_ASSERT( scheduler->currentTask().type == ResourceScheduler::SyncCollectionAttributes );
+  if ( job->error() ) {
+    emit q->error( QLatin1String( "Error while updating collection: " ) + job->errorString() );
+  }
+  emit q->attributesSynchronized( scheduler->currentTask().collection.id() );
+  scheduler->taskDone();
+}
+
 void ResourceBasePrivate::slotDeleteResourceCollection()
 {
   Q_Q( ResourceBase );
@@ -326,7 +416,7 @@ void ResourceBase::changeCommitted( const Item& item )
   ItemModifyJob *job = new ItemModifyJob( item );
   job->d_func()->setClean();
   job->disableRevisionCheck(); // TODO: remove, but where/how do we handle the error?
-  job->ignorePayload(); // we only want to reset the dirty flag and update the remote id
+  job->setIgnorePayload( true ); // we only want to reset the dirty flag and update the remote id
   d->changeProcessed();
 }
 
@@ -485,6 +575,12 @@ void ResourceBasePrivate::slotSynchronizeCollection( const Collection &col )
   scheduler->taskDone();
 }
 
+void ResourceBasePrivate::slotSynchronizeCollectionAttributes( const Collection &col )
+{
+  Q_Q( ResourceBase );
+  QMetaObject::invokeMethod( q, "retrieveCollectionAttributes", Q_ARG( Akonadi::Collection, col ) );
+}
+
 void ResourceBasePrivate::slotPrepareItemRetrieval( const Akonadi::Item &item )
 {
   Q_Q( ResourceBase );
@@ -611,19 +707,50 @@ void ResourceBase::doSetOnline( bool state )
 
 void ResourceBase::synchronizeCollection( qint64 collectionId )
 {
-  CollectionFetchJob* job = new CollectionFetchJob( Collection( collectionId ), CollectionFetchJob::Base );
+  synchronizeCollection( collectionId, false );
+}
+
+void ResourceBase::synchronizeCollection( qint64 collectionId, bool recursive )
+{
+  CollectionFetchJob* job = new CollectionFetchJob( Collection( collectionId ), recursive ? CollectionFetchJob::Recursive : CollectionFetchJob::Base );
   job->setFetchScope( changeRecorder()->collectionFetchScope() );
   job->fetchScope().setResource( identifier() );
+  job->setProperty( "recursive", recursive );
   connect( job, SIGNAL( result( KJob* ) ), SLOT( slotCollectionListDone( KJob* ) ) );
 }
 
 void ResourceBasePrivate::slotCollectionListDone( KJob *job )
+{
+  if ( !job->error() ) {
+    Collection::List list = static_cast<CollectionFetchJob*>( job )->collections();
+    if ( !list.isEmpty() ) {
+      if ( job->property( "recursive" ).toBool() ) {
+        Q_FOREACH ( const Collection &collection, list ) {
+          scheduler->scheduleSync( collection );
+        }
+      } else {
+        scheduler->scheduleSync( list.first() );
+      }
+    }
+  }
+  // TODO: error handling
+}
+
+void ResourceBase::synchronizeCollectionAttributes( qint64 collectionId )
+{
+  CollectionFetchJob* job = new CollectionFetchJob( Collection( collectionId ), CollectionFetchJob::Base );
+  job->setFetchScope( changeRecorder()->collectionFetchScope() );
+  job->fetchScope().setResource( identifier() );
+  connect( job, SIGNAL( result( KJob* ) ), SLOT( slotCollectionListForAttributesDone( KJob* ) ) );
+}
+
+void ResourceBasePrivate::slotCollectionListForAttributesDone( KJob *job )
 {
   if ( !job->error() ) {
     Collection::List list = static_cast<CollectionFetchJob*>( job )->collections();
     if ( !list.isEmpty() ) {
       Collection col = list.first();
-      scheduler->scheduleSync( col );
+      scheduler->scheduleAttributesSync( col );
     }
   }
   // TODO: error handling
@@ -640,42 +767,21 @@ void ResourceBase::setTotalItems( int amount )
 void ResourceBase::setItemStreamingEnabled( bool enable )
 {
   Q_D( ResourceBase );
-  Q_ASSERT_X( d->scheduler->currentTask().type == ResourceScheduler::SyncCollection,
-              "ResourceBase::setItemStreamingEnabled()",
-              "Calling setItemStreamingEnabled() although no item retrieval is in progress" );
-  if ( !d->mItemSyncer ) {
-    d->mItemSyncer = new ItemSync( currentCollection() );
-    connect( d->mItemSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
-    connect( d->mItemSyncer, SIGNAL( result( KJob* ) ), SLOT( slotItemSyncDone( KJob* ) ) );
-  }
+  d->createItemSyncInstanceIfMissing();
   d->mItemSyncer->setStreamingEnabled( enable );
 }
 
 void ResourceBase::itemsRetrieved( const Item::List &items )
 {
   Q_D( ResourceBase );
-  Q_ASSERT_X( d->scheduler->currentTask().type == ResourceScheduler::SyncCollection,
-              "ResourceBase::itemsRetrieved()",
-              "Calling itemsRetrieved() although no item retrieval is in progress" );
-  if ( !d->mItemSyncer ) {
-    d->mItemSyncer = new ItemSync( currentCollection() );
-    connect( d->mItemSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
-    connect( d->mItemSyncer, SIGNAL( result( KJob* ) ), SLOT( slotItemSyncDone( KJob* ) ) );
-  }
+  d->createItemSyncInstanceIfMissing();
   d->mItemSyncer->setFullSyncItems( items );
 }
 
 void ResourceBase::itemsRetrievedIncremental( const Item::List &changedItems, const Item::List &removedItems )
 {
   Q_D( ResourceBase );
-  Q_ASSERT_X( d->scheduler->currentTask().type == ResourceScheduler::SyncCollection,
-              "ResourceBase::itemsRetrievedIncremental()",
-              "Calling itemsRetrievedIncremental() although no item retrieval is in progress" );
-  if ( !d->mItemSyncer ) {
-    d->mItemSyncer = new ItemSync( currentCollection() );
-    connect( d->mItemSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
-    connect( d->mItemSyncer, SIGNAL( result( KJob* ) ), SLOT( slotItemSyncDone( KJob* ) ) );
-  }
+  d->createItemSyncInstanceIfMissing();
   d->mItemSyncer->setIncrementalSyncItems( changedItems, removedItems );
 }
 
@@ -689,11 +795,41 @@ void ResourceBasePrivate::slotItemSyncDone( KJob *job )
   scheduler->taskDone();
 }
 
+
+void ResourceBasePrivate::slotDelayedEmitProgress()
+{
+  Q_Q( ResourceBase );
+  emit q->percent( mUnemittedProgress );
+
+  Q_FOREACH( const QVariantMap &statusMap, mUnemittedAdvancedStatus ) {
+      emit q->advancedStatus( statusMap );
+  }
+  mUnemittedProgress = 0;
+  mUnemittedAdvancedStatus.clear();
+}
+
 void ResourceBasePrivate::slotPercent( KJob *job, unsigned long percent )
 {
   Q_Q( ResourceBase );
-  Q_UNUSED( job );
-  emit q->percent( percent );
+
+  mUnemittedProgress = percent;
+
+  const Collection collection = job->property( "collection" ).value<Collection>();
+  if ( collection.isValid() ) {
+    QVariantMap statusMap;
+    statusMap.insert( QLatin1String( "key" ), QString::fromLatin1( "collectionSyncProgress" ) );
+    statusMap.insert( QLatin1String( "collectionId" ), collection.id() );
+    statusMap.insert( QLatin1String( "percent" ), static_cast<unsigned int>( percent ) );
+
+    mUnemittedAdvancedStatus[collection.id()] = statusMap;
+  }
+  // deliver completion right away, intermediate progress at 1s intervals
+  if ( percent == 100 ) {
+    mProgressEmissionCompressor.stop();
+    slotDelayedEmitProgress();
+  } else if ( !mProgressEmissionCompressor.isActive() ) {
+    mProgressEmissionCompressor.start();
+  }
 }
 
 void ResourceBase::setHierarchicalRemoteIdentifiersEnabled( bool enable )
@@ -714,5 +850,29 @@ void ResourceBase::taskDone()
   d->scheduler->taskDone();
 }
 
+void ResourceBase::retrieveCollectionAttributes( const Collection &collection )
+{
+  collectionAttributesRetrieved( collection );
+}
+
+void Akonadi::ResourceBase::abortActivity()
+{
+
+}
+
+void ResourceBase::setItemTransactionMode(ItemSync::TransactionMode mode)
+{
+  Q_D( ResourceBase );
+  d->mItemTransactionMode = mode;
+}
+
+void ResourceBase::setItemSynchronizationFetchScope(const ItemFetchScope& fetchScope)
+{
+  Q_D( ResourceBase );
+  if ( !d->mItemSyncFetchScope )
+    d->mItemSyncFetchScope = new ItemFetchScope;
+  *(d->mItemSyncFetchScope) = fetchScope;
+}
+
 #include "resourcebase.moc"
 #include "moc_resourcebase.cpp"
