@@ -19,6 +19,8 @@
 
 #include "resourcescheduler_p.h"
 
+#include "dbusconnectionpool.h"
+
 #include <kdebug.h>
 #include <klocale.h>
 
@@ -36,6 +38,7 @@ static QDBusAbstractInterface *s_resourcetracker = 0;
 
 ResourceScheduler::ResourceScheduler( QObject *parent ) :
     QObject( parent ),
+    mCurrentTasksQueue( -1 ),
     mOnline( false )
 {
 }
@@ -77,6 +80,20 @@ void ResourceScheduler::scheduleSync(const Collection & col)
   scheduleNext();
 }
 
+void ResourceScheduler::scheduleAttributesSync( const Collection &collection )
+{
+  Task t;
+  t.type = SyncCollectionAttributes;
+  t.collection = collection;
+
+  TaskList& queue = queueForTaskType( t.type );
+  if ( queue.contains( t ) || mCurrentTask == t )
+    return;
+  queue << t;
+  signalTaskToTracker( t, "SyncCollectionAttributes" );
+  scheduleNext();
+}
+
 void ResourceScheduler::scheduleItemFetch(const Item & item, const QSet<QByteArray> &parts, const QDBusMessage & msg)
 {
   Task t;
@@ -151,6 +168,8 @@ void Akonadi::ResourceScheduler::scheduleCustomTask( QObject *receiver, const ch
   QueueType queueType = GenericTaskQueue;
   if ( priority == ResourceBase::AfterChangeReplay )
     queueType = AfterChangeReplayQueue;
+  else if ( priority == ResourceBase::Prepend )
+    queueType = PrependTaskQueue;
   TaskList& queue = mTaskList[ queueType ];
 
   if ( queue.contains( t ) )
@@ -182,11 +201,15 @@ void ResourceScheduler::taskDone()
   }
 
   mCurrentTask = Task();
+  mCurrentTasksQueue = -1;
   scheduleNext();
 }
 
 void ResourceScheduler::deferTask()
 {
+  if ( mCurrentTask.type == Invalid )
+      return;
+
   if ( s_resourcetracker ) {
     QList<QVariant> argumentList;
     argumentList << QString::number( mCurrentTask.serial )
@@ -196,7 +219,11 @@ void ResourceScheduler::deferTask()
 
   Task t = mCurrentTask;
   mCurrentTask = Task();
-  mTaskList[GenericTaskQueue] << t;
+
+  Q_ASSERT( mCurrentTasksQueue >= 0 && mCurrentTasksQueue < NQueueCount );
+  mTaskList[mCurrentTasksQueue].prepend( t );
+  mCurrentTasksQueue = -1;
+
   signalTaskToTracker( t, "DeferedTask" );
 
   scheduleNext();
@@ -226,6 +253,7 @@ void ResourceScheduler::executeNext()
   for ( int i = 0; i < NQueueCount; ++i ) {
     if ( !mTaskList[ i ].isEmpty() ) {
       mCurrentTask = mTaskList[ i ].takeFirst();
+      mCurrentTasksQueue = i;
       break;
     }
   }
@@ -246,6 +274,9 @@ void ResourceScheduler::executeNext()
     case SyncCollection:
       emit executeCollectionSync( mCurrentTask.collection );
       break;
+    case SyncCollectionAttributes:
+      emit executeCollectionAttributesSync( mCurrentTask.collection );
+      break;
     case FetchItem:
       emit executeItemFetch( mCurrentTask.item, mCurrentTask.itemParts );
       break;
@@ -293,6 +324,7 @@ void ResourceScheduler::setOnline(bool state)
       // abort running task
       queueForTaskType( mCurrentTask.type ).prepend( mCurrentTask );
       mCurrentTask = Task();
+      mCurrentTasksQueue = -1;
     }
     // abort pending synchronous tasks, might take longer until the resource goes online again
     TaskList& itemFetchQueue = queueForTaskType( FetchItem );
@@ -316,11 +348,11 @@ void ResourceScheduler::setOnline(bool state)
 void ResourceScheduler::signalTaskToTracker( const Task &task, const QByteArray &taskType )
 {
   // if there's a job tracer running, tell it about the new job
-  if ( !s_resourcetracker && QDBusConnection::sessionBus().interface()->isServiceRegistered(QLatin1String( "org.kde.akonadiconsole" ) ) ) {
+  if ( !s_resourcetracker && DBusConnectionPool::threadConnection().interface()->isServiceRegistered(QLatin1String( "org.kde.akonadiconsole" ) ) ) {
     s_resourcetracker = new QDBusInterface( QLatin1String( "org.kde.akonadiconsole" ),
                                        QLatin1String( "/resourcesJobtracker" ),
                                        QLatin1String( "org.freedesktop.Akonadi.JobTracker" ),
-                                       QDBusConnection::sessionBus(), 0 );
+                                       DBusConnectionPool::threadConnection(), 0 );
   }
 
   if ( s_resourcetracker ) {
@@ -352,7 +384,7 @@ void ResourceScheduler::Task::sendDBusReplies( bool success )
   Q_FOREACH( const QDBusMessage &msg, dbusMsgs ) {
     QDBusMessage reply( msg );
     reply << success;
-    QDBusConnection::sessionBus().send( reply );
+    DBusConnectionPool::threadConnection().send( reply );
   }
 }
 
@@ -395,6 +427,22 @@ void ResourceScheduler::clear()
     queue.clear();
   }
   mCurrentTask = Task();
+  mCurrentTasksQueue = -1;
+}
+
+void Akonadi::ResourceScheduler::cancelQueues()
+{
+  for ( int i = 0; i < NQueueCount; ++i ) {
+    TaskList& queue = mTaskList[i];
+    if ( s_resourcetracker ) {
+      foreach ( const Task &t, queue ) {
+        QList<QVariant> argumentList;
+        argumentList << QString::number( t.serial ) << QString();
+        s_resourcetracker->asyncCallWithArgumentList(QLatin1String( "jobEnded" ), argumentList);
+      }
+    }
+    queue.clear();
+  }
 }
 
 static const char s_taskTypes[][25] = {
