@@ -50,6 +50,7 @@ class ResourceScheduler : public QObject
       SyncAll,
       SyncCollectionTree,
       SyncCollection,
+      SyncCollectionAttributes,
       FetchItem,
       ChangeReplay,
       DeleteResourceCollection,
@@ -104,6 +105,12 @@ class ResourceScheduler : public QObject
     */
     void scheduleSync( const Collection &col );
 
+    /**
+      Schedules synchronizing the attributes of a single collection.
+      @param collection The collection to synchronize attributes from.
+    */
+    void scheduleAttributesSync( const Collection &collection );
+
     /**
       Schedules fetching of a single PIM item.
       @param item The item to fetch.
@@ -156,6 +163,13 @@ class ResourceScheduler : public QObject
     */
     void clear();
 
+    /**
+       Cancel everything the scheduler has still in queue. Keep the current task running though.
+       It can be seen as a less aggressive clear() used when the user requested the resource to
+       abort its activities. It properly cancel all the tasks in there.
+    */
+    void cancelQueues();
+
   public Q_SLOTS:
     /**
       Schedules replaying changes.
@@ -179,6 +193,7 @@ class ResourceScheduler : public QObject
 
   Q_SIGNALS:
     void executeFullSync();
+    void executeCollectionAttributesSync( const Akonadi::Collection &col );
     void executeCollectionSync( const Akonadi::Collection &col );
     void executeCollectionTreeSync();
     void executeItemFetch( const Akonadi::Item &item, const QSet<QByteArray> &parts );
