@@ -56,7 +56,6 @@ imap://server/folder/
  *  ERR_DOES_NOT_EXIST is reserved for folders.
  */
 
-#include <kdepimlibs-compat.h> // for KDE_signal, remove in KDEPIM 4.2
 #include "imap4.h"
 
 #include <QByteArray>
@@ -83,7 +82,6 @@ extern "C" {
 #include <kdebug.h>
 #include <kio/connection.h>
 #include <kio/slaveinterface.h>
-#include <kio/passworddialog.h>
 #include <klocale.h>
 #include <kmimetype.h>
 #include <kcodecs.h>
@@ -883,7 +881,7 @@ IMAP4Protocol::put (const KUrl & _url, int, KIO::JobFlags)
           QString uid = cmd->resultInfo();
           if ( uid.contains("APPENDUID") )
           {
-            uid = uid.section(" ", 2, 2);
+            uid = uid.section(' ', 2, 2);
             uid.truncate(uid.length()-1);
             infoMessage("UID "+uid);
           }
@@ -1057,7 +1055,7 @@ IMAP4Protocol::copy (const KUrl & src, const KUrl & dest, int, KIO::JobFlags fla
         QString uid = cmd->resultInfo();
         if ( uid.contains("COPYUID") )
         {
-          uid = uid.section(" ", 2, 3);
+          uid = uid.section(' ', 2, 3);
           uid.truncate(uid.length()-1);
           infoMessage("UID "+uid);
         }
@@ -1409,7 +1407,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     if (cmd->result () != "OK")
     {
       completeQueue.removeAll (cmd);
-      error(ERR_COULD_NOT_WRITE,
+      error(ERR_SLAVE_DEFINED,
             i18n( "Changing the flags of message %1 failed.", _url.prettyUrl() ) );
       return;
     }
@@ -1784,7 +1782,7 @@ IMAP4Protocol::rename (const KUrl & src, const KUrl & dest, KIO::JobFlags flags)
           completeQueue.removeAll(cmd);
           if (!ok)
           {
-            error(ERR_CANNOT_RENAME, i18n("Unable to close mailbox."));
+            error(ERR_SLAVE_DEFINED, i18n("Unable to close mailbox."));
             return;
           }
           setState(ISTATE_LOGIN);
@@ -1852,7 +1850,7 @@ IMAP4Protocol::stat (const KUrl & _url)
       completeQueue.removeAll(cmd);
       if (!ok)
       {
-        error(ERR_COULD_NOT_STAT, i18n("Unable to close mailbox."));
+        error(ERR_SLAVE_DEFINED, i18n("Unable to close mailbox."));
         return;
       }
       setState(ISTATE_LOGIN);
@@ -1882,7 +1880,7 @@ IMAP4Protocol::stat (const KUrl & _url)
       }
       completeQueue.removeAll (cmd);
       if (found)
-        error(ERR_COULD_NOT_STAT, i18n("Unable to get information about folder %1. The server replied: %2", aBox, cmdInfo));
+        error(ERR_SLAVE_DEFINED, i18n("Unable to get information about folder %1. The server replied: %2", aBox, cmdInfo));
       else
         error(KIO::ERR_DOES_NOT_EXIST, aBox);
       return;
@@ -2125,13 +2123,13 @@ bool IMAP4Protocol::makeLogin ()
         }
       }
       if (!clientLogin (myUser, myPass, resultInfo))
-        error(KIO::ERR_COULD_NOT_AUTHENTICATE, i18n("Unable to login. Probably the "
+        error(ERR_SLAVE_DEFINED, i18n("Unable to login. Probably the "
         "password is wrong.\nThe server %1 replied:\n%2", myHost, resultInfo));
     }
     else
     {
       if (!clientAuthenticate (this, authInfo, myHost, myAuth, mySSL, resultInfo))
-        error(KIO::ERR_COULD_NOT_AUTHENTICATE, i18n("Unable to authenticate via %1.\n"	"The server %2 replied:\n%3", myAuth, myHost, resultInfo));
+        error(ERR_SLAVE_DEFINED, i18n("Unable to authenticate via %1.\n"	"The server %2 replied:\n%3", myAuth, myHost, resultInfo));
       else {
         myUser = authInfo.username;
         myPass = authInfo.password;
@@ -2501,7 +2499,7 @@ IMAP4Protocol::parseURL (const KUrl & _url, QString & _box,
         << "from URL" << _url.path();
     }
     if (_hierarchyDelimiter.isEmpty())
-      _hierarchyDelimiter = "/";
+      _hierarchyDelimiter = '/';
   }
   kDebug(7116) <<"IMAP4::parseURL - return" << retVal;
 
@@ -2549,9 +2547,9 @@ void IMAP4Protocol::flushOutput(const QString &contentEncoding)
   {
     // get the coding from the MIME header
     QByteArray decoded;
-    if ( contentEncoding.startsWith("quoted-printable", Qt::CaseInsensitive) )
+    if ( contentEncoding.startsWith(QLatin1String("quoted-printable"), Qt::CaseInsensitive) )
       decoded = KCodecs::quotedPrintableDecode(outputCache);
-    else if ( contentEncoding.startsWith("base64", Qt::CaseInsensitive) )
+    else if ( contentEncoding.startsWith(QLatin1String("base64"), Qt::CaseInsensitive) )
       decoded = QByteArray::fromBase64( outputCache );
     else
       decoded = outputCache;
